@@ -6,7 +6,6 @@ from __future__ import (absolute_import, division, print_function,
                         with_statement)
 
 import ast
-from collections import Sequence
 import contextlib
 import copy
 import six
@@ -15,6 +14,13 @@ from   six.moves                import builtins
 import sys
 import types
 
+if PY3:
+    from collections.abc import Sequence
+else:
+    # This is deprecated in Python 3
+    from collections import Sequence
+
+
 from   pyflyby._file            import FileText, Filename
 from   pyflyby._flags           import CompilerFlags
 from   pyflyby._idents          import (BadDottedIdentifierError,
@@ -420,6 +426,11 @@ class _MissingImportFinder(object):
                 self.visit(item)
         elif isinstance(node, ast.AST):
             method = 'visit_' + node.__class__.__name__
+            if not hasattr(self, method):
+                logger.debug(
+                    "_MissingImportFinder has no method %r, using generic_visit", method
+                )
+
             visitor = getattr(self, method, self.generic_visit)
             return visitor(node)
         else:
@@ -522,6 +533,9 @@ class _MissingImportFinder(object):
         # class).
         self._visit_Store(node.name)
 
+    def visit_AsyncFunctionDef(self, node):
+        return self.visit_FunctionDef(node)
+
     def visit_FunctionDef(self, node):
         # Visit a function definition.
         #   - Visit args and decorator list normally.
@@ -840,8 +854,10 @@ class _MissingImportFinder(object):
             # On the other hand, we intentionally alias the other scopes
             # rather than cloning them, because the point is to allow them to
             # be modified until we do the check at the end.
-            data = (fullname, self.scopestack.clone_top(), self._lineno)
-            self._deferred_load_checks.append(data)
+
+            if symbol_needs_import(fullname, self.scopestack):
+                data = (fullname, self.scopestack.clone_top(), self._lineno)
+                self._deferred_load_checks.append(data)
         else:
             # We're not in a FunctionDef.  Deferring would give us the same
             # result; we do the check now to avoid the overhead of cloning the
@@ -1821,7 +1837,8 @@ def auto_eval(arg, filename=None, mode=None,
     :return:
       Result of evaluation (for mode="eval")
     """
-    flags = CompilerFlags(flags)
+    if isinstance(flags, int):
+        assert isinstance(flags, CompilerFlags)
     if isinstance(arg, (six.string_types, Filename, FileText, PythonBlock)):
         block = PythonBlock(arg, filename=filename, flags=flags,
                             auto_flags=auto_flags)
